"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.init = void 0;
const errors_1 = require("@oclif/errors");
const tree_1 = require("../tree");
// deps in Help#topics
const stripAnsi = require("strip-ansi").stripAnsi;
const { compact } = require("@oclif/plugin-help/lib/util");
const wrap = require("wrap-ansi");
// deps in Help#topic
const help = require("@oclif/plugin-help").default;
const renderList = require("@oclif/plugin-help/lib/list").renderList;
const chalk = require("chalk").default;
const indent = require("indent-string");
const { bold } = chalk;
const init = async function (ctx) {
    // build tree
    let cmds = ctx.config.commandIDs;
    let tree = new tree_1.default();
    cmds.forEach((c) => {
        let bits = c.split(":");
        let cur = tree;
        bits.forEach((b) => {
            cur = cur.findOrInsert(b);
        });
    });
    let id = (typeof ctx.id === "string" ? [ctx.id] : ctx.id) || [];
    let RAWARGV = id.concat(ctx.argv || []);
    const convertName = function (id) {
        return id.join(":");
    };
    const convertArgv = function (id, old = process.argv) {
        let keys = id.split(":");
        let argv = old.slice(keys.length + 2, old.length);
        return argv;
    };
    // overwrite config.findCommand
    const findCommand = ctx.config.findCommand;
    function spacesFindCommand(_, __) {
        let [node, c] = tree.findMostProgressiveCmd(RAWARGV);
        if (node) {
            // commented to support "index.js" when no sub-command is given
            // if (Object.keys((node as CommandTree).nodes).length) return;
            return findCommand.apply(ctx.config, [convertName(c)]);
        }
        return;
    }
    ctx.config.findCommand = spacesFindCommand;
    // overwrite config.findTopic
    const findTopic = ctx.config.findTopic;
    function spacesFindTopic(_, __) {
        let [node, c] = tree.findMostProgressiveCmd(RAWARGV);
        if (node) {
            return findTopic.apply(ctx.config, [convertName(c)]);
        }
        return;
    }
    ctx.config.findTopic = spacesFindTopic;
    // overwrite config.runCommand
    ctx.config.runCommand = async (id, argv = []) => {
        // tslint:disable-next-line:no-unused
        let [_, name] = tree.findMostProgressiveCmd(RAWARGV);
        // override the id b/c of the closure
        id = name.join(" ");
        argv = convertArgv(name.join(":"));
        // don't need to pass ID b/c of the closure
        const c = ctx.config.findCommand("");
        if (!c) {
            await ctx.config.runHook("command_not_found", { id });
            throw new errors_1.CLIError(`command ${id} not found`);
        }
        const command = c.load();
        await ctx.config.runHook("prerun", { Command: command, argv });
        await command.run(argv, ctx.config);
    };
    // overwrite Help#topics
    help.prototype.topics = function (topics) {
        if (!topics.length)
            return;
        let body = renderList(topics.map((c) => [
            c.name.replace(/:/g, " "),
            c.description && this.render(c.description.split("\n")[0]),
        ]), {
            spacer: "\n",
            stripAnsi: this.opts.stripAnsi,
            maxWidth: this.opts.maxWidth - 2,
        });
        return [bold("COMMANDS"), indent(body, 2)].join("\n");
    };
    // overwrite Help#topic
    help.prototype.topic = function (topic) {
        let description = this.render(topic.description || "");
        let title = description.split("\n")[0];
        description = description.split("\n").slice(1).join("\n");
        let output = compact([
            title,
            [
                bold("USAGE"),
                indent(wrap(`$ ${this.config.bin} ${topic.name.replace(/:/g, " ")} COMMAND`, this.opts.maxWidth - 2, { trim: false, hard: true }), 2),
            ].join("\n"),
            description &&
                [
                    bold("DESCRIPTION"),
                    indent(wrap(description, this.opts.maxWidth - 2, {
                        trim: false,
                        hard: true,
                    }), 2),
                ].join("\n"),
        ]).join("\n\n");
        if (this.opts.stripAnsi)
            output = stripAnsi(output);
        return output + "\n";
    };
};
exports.init = init;
